DEFAULT_OBJECT_DIRECTORY = "models"
KEY_NAME = "name"
ELEMENT_INDEX = "index"
TOKENS = "tokens"
INTENT_NAME_KEY = "name"
PREDICTED_CONFIDENCE_KEY = "confidence"
ENTITIES = "entities"
INPUT_IDS = "input_ids"
ATTENTION_MASKS = "attention_masks"
# PIPELINE

KEY_PIPELINE = "pipeline"
KEY_LANGUAGE = "language"

DEFAULT_ENCODING = "utf-8"
